@@ -1,18 +1,30 @@
 
-local rawset, next, table, pairs, print, require, io, os, setmetatable, pcall, ipairs, package =
-      rawset, next, table, pairs, print, require, io, os, setmetatable, pcall, ipairs, package
+local rawset, next, table, pairs, print, require, io, os, setmetatable, pcall, ipairs, package, type =
+      rawset, next, table, pairs, print, require, io, os, setmetatable, pcall, ipairs, package, type
 
 --- Configuration for LuaRocks.
 -- Tries to load the user's configuration file and
 -- defines defaults for unset values. See the
 -- <a href="http://luarocks.org/en/Config_file_format">config
 -- file format documentation</a> for details.
+--
+-- End-users shouldn't edit this file. They can override any defaults
+-- set in this file using their system-wide $LUAROCKS_SYSCONFIG file
+-- (see luarocks.config) or their user-specific configuration file
+-- (~/.luarocks/config.lua on Unix or %APPDATA%/luarocks/config.lua on
+-- Windows).
 module("luarocks.cfg")
 
 -- Load site-local global configurations
-local config = require("luarocks.config")
+local ok, config = pcall(require, "luarocks.config")
+if not ok then
+   print("Site-local luarocks/config.lua file not found. Incomplete installation?")
+   config = {}
+end
 
-program_version = "2.0.1"
+_M.config = config
+
+program_version = "2.0.2"
 user_agent = "LuaRocks/"..program_version
 
 local persist = require("luarocks.persist")
@@ -62,6 +74,8 @@ elseif system and system:match("^CYGWIN") then
    detected.cygwin = true
 elseif system and system:match("^Windows") then
    detected.windows = true
+elseif system and system:match("^MINGW") then
+   detected.mingw32 = true
 else
    detected.unix = true
    -- Fall back to Unix in unknown systems.
@@ -70,7 +84,7 @@ end
 -- Path configuration:
 
 local sys_config_file, home_config_file, home_tree
-if detected.windows then
+if detected.windows or detected.mingw32 then
    home = os.getenv("APPDATA") or "c:"
    sys_config_file = "c:/luarocks/config.lua"
    home_config_file = home.."/luarocks/config.lua"
@@ -107,15 +121,10 @@ end
 
 -- Configure defaults:
 
-local root = config.LUAROCKS_ROCKS_TREE or home_tree
+local root = rocks_trees[#rocks_trees]
 local defaults = {
-   root_dir = root,
-   rocks_dir = root.."/lib/luarocks/rocks",
    lua_modules_path = "/share/lua/5.1/",
    lib_modules_path = "/lib/lua/5.1/",
-   deploy_bin_dir = root.."/bin/",
-   deploy_lua_dir = root.."/share/lua/5.1/",
-   deploy_lib_dir = root.."/lib/lua/5.1/",
 
    arch = "unknown",
    lib_extension = "unknown",
@@ -159,18 +168,53 @@ if detected.windows then
    defaults.make = "nmake" -- TODO: Split Windows flavors between mingw and msvc
    defaults.makefile = "Makefile.win"
    defaults.variables.CC = "cl"
+   defaults.variables.RC = "rc"
+   defaults.variables.WRAPPER = config.LUAROCKS_PREFIX .. "\\2.0\\rclauncher.obj"
    defaults.variables.LD = "link"
    defaults.variables.MT = "mt"
    defaults.variables.CFLAGS = "/MD /O2"
    defaults.variables.LIBFLAG = "/dll"
    defaults.external_deps_patterns = {
       bin = { "?.exe", "?.bat" },
-      lib = { "?.lib", "?.dll" },
+      lib = { "?.lib", "?.dll", "lib?.dll" },
       include = { "?.h" }
    }
    defaults.runtime_external_deps_patterns = {
       bin = { "?.exe", "?.bat" },
-      lib = { "?.dll" },
+      lib = { "?.dll", "lib?.dll" },
+      include = { "?.h" }
+   }
+   defaults.local_cache = home.."/cache/luarocks"
+end
+
+if detected.mingw32 then
+   home_config_file = home_config_file:gsub("\\","/")
+   defaults.arch = "win32-"..proc
+   defaults.platforms = { "win32", "mingw32" }
+   defaults.lib_extension = "dll"
+   defaults.external_lib_extension = "dll"
+   defaults.obj_extension = "o"
+   defaults.external_deps_dirs = { "c:/external/" }
+   defaults.variables.LUA_BINDIR = config.LUA_BINDIR and config.LUA_BINDIR:gsub("\\", "/") or "c:/lua5.1/bin"
+   defaults.variables.LUA_INCDIR = config.LUA_INCDIR and config.LUA_INCDIR:gsub("\\", "/") or "c:/lua5.1/include"
+   defaults.variables.LUA_LIBDIR = config.LUA_LIBDIR and config.LUA_LIBDIR:gsub("\\", "/") or "c:/lua5.1/lib"
+   defaults.cmake_generator = "MinGW Makefiles"
+   defaults.make = "mingw32-make" -- TODO: Split Windows flavors between mingw and msvc
+   defaults.makefile = "Makefile.win"
+   defaults.variables.CC = "mingw32-gcc"
+   defaults.variables.RC = "windres"
+   defaults.variables.WRAPPER = config.LUAROCKS_PREFIX .. "\\2.0\\rclauncher.o"
+   defaults.variables.LD = "mingw32-gcc"
+   defaults.variables.CFLAGS = "-O2"
+   defaults.variables.LIBFLAG = "-shared"
+   defaults.external_deps_patterns = {
+      bin = { "?.exe", "?.bat" },
+      lib = { "?.lib", "?.dll", "lib?.dll" },
+      include = { "?.h" }
+   }
+   defaults.runtime_external_deps_patterns = {
+      bin = { "?.exe", "?.bat" },
+      lib = { "?.dll", "lib?.dll" },
       include = { "?.h" }
    }
    defaults.local_cache = home.."/cache/luarocks"
@@ -277,7 +321,13 @@ setmetatable(_M, cfg_mt)
 
 
 for _,tree in ipairs(rocks_trees) do
-   package.path = tree..lua_modules_path.."/?.lua;"..tree..lua_modules_path.."/?/init.lua;"..package.path
-   package.cpath = tree..lib_modules_path.."/?."..lib_extension..";"..package.cpath
+  if type(tree) == "string" then
+    package.path = tree..lua_modules_path.."/?.lua;"..tree..lua_modules_path.."/?/init.lua;"..package.path
+    package.cpath = tree..lib_modules_path.."/?."..lib_extension..";"..package.cpath
+  else
+    package.path = (tree.lua_dir or tree.root..lua_modules_path).."/?.lua;"..
+                       (tree.lua_dir or tree.root..lua_modules_path).."/?/init.lua;"..package.path
+    package.cpath = (tree.lib_dir or tree.root..lib_modules_path).."/?."..lib_extension..";"..package.cpath
+  end
 end
 
