@@ -7,10 +7,14 @@ local fs = require("luarocks.fs")
 
 local cfg = require("luarocks.cfg")
 local dir = require("luarocks.dir")
+local util = require("luarocks.util")
 
-local zip_ok, zip = pcall(require, "luarocks.tools.zip")
+local socket_ok, http = pcall(require, "socket.http")
+local _, ftp = pcall(require, "socket.ftp")
+local zip_ok, lrzip = pcall(require, "luarocks.tools.zip")
+local unzip_ok, luazip = pcall(require, "zip"); _G.zip = nil
 local lfs_ok, lfs = pcall(require, "lfs")
-local curl_ok, curl = pcall(require, "luacurl")
+--local curl_ok, curl = pcall(require, "luacurl")
 local md5_ok, md5 = pcall(require, "md5")
 local posix_ok, posix = pcall(require, "posix")
 
@@ -18,7 +22,7 @@ local tar = require("luarocks.tools.tar")
 local patch = require("luarocks.tools.patch")
 
 local dir_stack = {}
-            
+
 math.randomseed(os.time())
 
 dir_separator = "/"
@@ -29,7 +33,7 @@ dir_separator = "/"
 -- @return string: Quoted argument.
 function Q(arg)
    assert(type(arg) == "string")
-   
+
    -- FIXME Unix-specific
    return "'" .. arg:gsub("\\", "\\\\"):gsub("'", "'\\''") .. "'"
 end
@@ -45,12 +49,12 @@ function is_writable(file)
    local result
    if fs.is_dir(file) then
       local file2 = file .. '/.tmpluarockstestwritable'
-      local fh = io.open(file2, 'w')
+      local fh = io.open(file2, 'wb')
       result = fh ~= nil
       if fh then fh:close() end
       os.remove(file2)
    else
-      local fh = io.open(file, 'r+')
+      local fh = io.open(file, 'rb+')
       result = fh ~= nil
       if fh then fh:close() end
    end
@@ -64,6 +68,7 @@ end
 function make_temp_dir(name)
    assert(type(name) == "string")
 
+   name = name:gsub("\\", "/")
    local temp_dir = (os.getenv("TMP") or "/tmp") .. "/luarocks_" .. name:gsub(dir.separator, "_") .. "-" .. tostring(math.floor(math.random() * 10000))
    if fs.make_dir(temp_dir) then
       return temp_dir
@@ -81,7 +86,7 @@ end
 -- otherwise.
 function execute(command, ...)
    assert(type(command) == "string")
-   
+
    for _, arg in ipairs({...}) do
       assert(type(arg) == "string")
       command = command .. " " .. fs.Q(arg)
@@ -170,9 +175,18 @@ end
 -- @return boolean: true on success, false on failure.
 function make_dir(directory)
    assert(type(directory) == "string")
+   directory = directory:gsub("\\", "/")
    local path = nil
-   for d in directory:gmatch("[^"..dir.separator.."]*"..dir.separator.."*") do
-      path = path and path..d or d
+   if directory:sub(2, 2) == ":" then
+     path = directory:sub(1, 2)
+     directory = directory:sub(4)
+   else
+     if directory:match("^/") then
+        path = ""
+     end
+   end
+   for d in directory:gmatch("([^"..dir.separator.."]+)"..dir.separator.."*") do
+      path = path and path .. dir.separator .. d or d
       local mode = lfs.attributes(path, "mode")
       if not mode then
          if not lfs.mkdir(path) then
@@ -217,9 +231,9 @@ function copy(src, dest)
    if destmode == "directory" then
       dest = dir.path(dest, dir.base_name(src))
    end
-   local src_h, err = io.open(src, "r")
+   local src_h, err = io.open(src, "rb")
    if not src_h then return nil, err end
-   local dest_h, err = io.open(dest, "w+")
+   local dest_h, err = io.open(dest, "wb+")
    if not dest_h then src_h:close() return nil, err end
    while true do
       local block = src_h:read(8192)
@@ -237,7 +251,7 @@ end
 -- @return boolean or (boolean, string): true on success, false on failure
 local function recursive_copy(src, dest)
    local srcmode = lfs.attributes(src, "mode")
-      
+
    if srcmode == "file" then
       local ok = fs.copy(src, dest)
       if not ok then return false end
@@ -262,11 +276,13 @@ end
 function copy_contents(src, dest)
    assert(src and dest)
    assert(lfs.attributes(src, "mode") == "directory")
-   
+
    for file in lfs.dir(src) do
-      local ok = recursive_copy(dir.path(src, file), dest)
-      if not ok then
-         return false, "Failed copying "..src.." to "..dest
+      if file ~= "." and file ~= ".." then
+         local ok = recursive_copy(dir.path(src, file), dest)
+         if not ok then
+            return false, "Failed copying "..src.." to "..dest
+         end
       end
    end
    return true
@@ -279,7 +295,7 @@ end
 -- or nil and an error message on failure.
 local function recursive_delete(src)
    local srcmode = lfs.attributes(src, "mode")
-      
+
    if srcmode == "file" then
       return os.remove(src)
    elseif srcmode == "directory" then
@@ -301,11 +317,10 @@ end
 -- @return boolean: true on success, false on failure.
 function delete(arg)
    assert(arg)
-   assert(arg:sub(1,1) == "/")
    return recursive_delete(arg) or false
 end
 
---- List the contents of a directory. 
+--- List the contents of a directory.
 -- @param at string or nil: directory to list (will be the current
 -- directory if none is given).
 -- @return table: an array of strings with the filenames representing
@@ -344,7 +359,7 @@ local function recursive_find(cwd, prefix, result)
    end
 end
 
---- Recursively scan the contents of a directory. 
+--- Recursively scan the contents of a directory.
 -- @param at string or nil: directory to scan (will be the current
 -- directory if none is given).
 -- @return table: an array of strings with the filenames representing
@@ -398,17 +413,38 @@ end
 
 if zip_ok then
 
-local function zip(zipfile, ...)
-   return zip.zip(zipfile, ...)
+function zip(zipfile, ...)
+   return lrzip.zip(zipfile, ...)
 end
 
+end
+
+if unzip_ok then
 --- Uncompress files from a .zip archive.
 -- @param zipfile string: pathname of .zip archive to be extracted.
 -- @return boolean: true on success, false on failure.
 function unzip(zipfile)
-   assert(zipfile)
-   -- FIXME!!!!
-   return fs.execute("unzip", zipfile)
+  local zipfile, err = luazip.open(zipfile)
+  if not zipfile then return nil, err end
+  local files = zipfile:files()
+  local file = files()
+  repeat
+	if file.filename:sub(#file.filename) == "/" then
+	  fs.make_dir(dir.path(fs.current_dir(), file.filename))
+	else
+      local rf, err = zipfile:open(file.filename)
+	  if not rf then zipfile:close(); return nil, err end
+	  local contents = rf:read("*a")
+	  rf:close()
+	  local wf, err = io.open(dir.path(fs.current_dir(), file.filename), "wb")
+	  if not wf then zipfile:close(); return nil, err end
+	  wf:write(contents)
+	  wf:close()
+	end
+	file = files()
+  until not file
+  zipfile:close()
+  return true
 end
 
 end
@@ -430,7 +466,7 @@ function download(url, filename)
    assert(type(url) == "string")
    assert(type(filename) == "string" or not filename)
 
-   filename = filename or dir.base_name(url)
+   filename = dir.path(fs.current_dir(), filename or dir.base_name(url))
 
    local c = curl.new()
    if not c then return false end
@@ -454,6 +490,50 @@ end
 
 end
 
+---------------------------------------------------------------------
+-- LuaSocket functions
+---------------------------------------------------------------------
+
+if socket_ok then
+
+--- Download a remote file.
+-- @param url string: URL to be fetched.
+-- @param filename string or nil: this function attempts to detect the
+-- resulting local filename of the remote file as the basename of the URL;
+-- if that is not correct (due to a redirection, for example), the local
+-- filename can be given explicitly as this second argument.
+-- @return boolean: true on success, false on failure.
+function download(url, filename)
+   assert(type(url) == "string")
+   assert(type(filename) == "string" or not filename)
+
+   filename = dir.path(fs.current_dir(), filename or dir.base_name(url))
+
+   local content, err
+   if util.starts_with(url, "http:") then
+      local res, status, headers, line = http.request(url)
+      if not res then
+         err = status
+      elseif status ~= 200 then
+         err = line
+      else
+         content = res
+      end
+   elseif util.starts_with(url, "ftp:") then
+      content, err = ftp.get(url)
+   end
+   if not content then
+      return false, "Failed downloading: " .. err
+   end
+   local file = io.open(filename, "wb")
+   if not file then return false end
+   file:write(content)
+   file:close()
+   return true
+end
+
+end
+
 ---------------------------------------------------------------------
 -- MD5 functions
 ---------------------------------------------------------------------
@@ -465,7 +545,7 @@ if md5_ok then
 -- @return string: The MD5 checksum
 function get_md5(file)
    file = fs.absolute_name(file)
-   local file = io.open(file, "r")
+   local file = io.open(file, "rb")
    if not file then return false end
    local computed = md5.sumhexa(file:read("*a"))
    file:close()
