@@ -22,17 +22,6 @@ function Q(arg)
    return '"' .. arg:gsub('"', '\\"') .. '"'
 end
 
---- Strip the last extension of a filename.
--- Example: "foo.tar.gz" becomes "foo.tar".
--- If filename has no dots, returns it unchanged.
--- @param filename string: The file name to strip.
--- @return string: The stripped name.
-local function strip_extension(filename)
-   assert(type(filename) == "string")
-
-   return (filename:gsub("%.[^.]+$", "")) or filename
-end
-
 --- Return an absolute pathname from a potentially relative one.
 -- @param pathname string: pathname to convert.
 -- @param relative_to string or nil: path to prepend when making
@@ -92,6 +81,7 @@ function copy_binary(filename, dest)
    end
    local exe_pattern = "%.[Ee][Xx][Ee]$"
    local base = dir.base_name(filename)
+   local dest = dir.dir_name(dest)
    if base:match(exe_pattern) then
       base = base:gsub(exe_pattern, ".lua")
       local helpname = dest.."/"..base
