@@ -23,51 +23,61 @@ or a filename of a locally available rock.
 
 --- Install a binary rock.
 -- @param rock_file string: local or remote filename of a rock.
--- @return boolean or (nil, string): True if succeeded or 
--- nil and an error message.
+-- @return boolean or (nil, string, [string]): True if succeeded or 
+-- nil and an error message and an optional error code.
 function install_binary_rock(rock_file)
+   assert(type(rock_file) == "string")
+
    local name, version, arch = path.parse_rock_name(rock_file)
    if not name then
       return nil, "Filename "..rock_file.." does not match format 'name-version-revision.arch.rock'."
    end
    if arch ~= "all" and arch ~= cfg.arch then
-      return nil, "Incompatible architecture "..arch
+      return nil, "Incompatible architecture "..arch, "arch"
    end
    if rep.is_installed(name, version) then
       rep.delete_version(name, version)
    end
+   
    local rollback = util.schedule_function(function()
       fs.delete(path.install_dir(name, version))
       fs.remove_dir_if_empty(path.versions_dir(name))
    end)
-   local ok, err = fetch.fetch_and_unpack_rock(rock_file, path.install_dir(name, version))
-   if not ok then return nil, err end
-   ok, err = rep.install_bins(name, version)
-
-   local rockspec, err = fetch.load_rockspec(path.rockspec_file(name, version))
+   
+   local ok, err, errcode = fetch.fetch_and_unpack_rock(rock_file, path.install_dir(name, version))
+   if not ok then return nil, err, errcode end
+   
+   local rockspec, err, errcode = fetch.load_rockspec(path.rockspec_file(name, version))
    if err then
-      return nil, "Failed loading rockspec for installed package: "..err
+      return nil, "Failed loading rockspec for installed package: "..err, errcode
    end
 
-   ok, err = deps.check_external_deps(rockspec, "install")
-   if err then
-      return nil, err
+   ok, err, errcode = deps.check_external_deps(rockspec, "install")
+   if err then return nil, err, errcode end
+
+   -- For compatibility with .rock files built with LuaRocks 1
+   if not fs.exists(path.rock_manifest_file(name, version)) then
+      ok, err = manif.make_rock_manifest(name, version)
+      if err then return nil, err end
    end
 
-   ok, err = deps.fulfill_dependencies(rockspec)
-   if err then
-      return nil, err
-   end
+   ok, err, errcode = deps.fulfill_dependencies(rockspec)
+   if err then return nil, err, errcode end
+
+   ok, err = rep.deploy_files(name, version)
+   if err then return nil, err end
+
+   util.remove_scheduled_function(rollback)
+   rollback = util.schedule_function(function()
+      rep.delete_version(name, version)
+   end)
 
    ok, err = rep.run_hook(rockspec, "post_install")
-   if err then
-      return nil, err
-   end
+   if err then return nil, err end
    
    ok, err = manif.update_manifest(name, version)
-   if err then
-      return nil, err
-   end
+   if err then return nil, err end
+   
    util.remove_scheduled_function(rollback)
    return true
 end
