@@ -4,6 +4,7 @@
 module("luarocks.make_manifest", package.seeall)
 
 local manif = require("luarocks.manif")
+local index = require("luarocks.index")
 local cfg = require("luarocks.cfg")
 
 help_summary = "Compile a manifest file for a repository."
@@ -23,10 +24,10 @@ function run(repo)
   
    print("Making manifest for "..repo)
    
-   ok = manif.make_manifest(repo)
+   local ok, err = manif.make_manifest(repo)
    if ok then
       print("Generating index.html for "..repo)
-      manif.make_index(repo)
+      index.make_index(repo)
    end
-   return ok
+   return ok, err
 end
